from __future__ import absolute_import, division, print_function

from collections import namedtuple

#--------------------------------------------------------------------------------
# Definitions
#--------------------------------------------------------------------------------

# a label and all meta information
Label = namedtuple( 'Label' , [

    'name'        , # The identifier of this label, e.g. 'car', 'person', ... .
                    # We use them to uniquely name a class

    'id'          , # An integer ID that is associated with this label.
                    # The IDs are used to represent the label in ground truth images
                    # An ID of -1 means that this label does not have an ID and thus
                    # is ignored when creating ground truth images (e.g. license plate).
                    # Do not modify these IDs, since exactly these IDs are expected by the
                    # evaluation server.

    'trainId'     , # Feel free to modify these IDs as suitable for your method. Then create
                    # ground truth images with train IDs, using the tools provided in the
                    # 'preparation' folder. However, make sure to validate or submit results
                    # to our evaluation server using the regular IDs above!
                    # For trainIds, multiple labels might have the same ID. Then, these labels
                    # are mapped to the same class in the ground truth images. For the inverse
                    # mapping, we use the label that is defined first in the list below.
                    # For example, mapping all void-type classes to the same ID in training,
                    # might make sense for some approaches.
                    # Max value is 255!

    'hasInstances', # Whether this label distinguishes between single instances or not

    'ignoreInEval', # Whether pixels having this class as ground truth label are ignored
                    # during evaluations or not

    'color'       , # The color of this label
    ] )


#--------------------------------------------------------------------------------
# A list of all labels
#--------------------------------------------------------------------------------

# Please adapt the train IDs as appropriate for your approach.
# Note that you might want to ignore labels with ID 255 during training.
# Further note that the current train IDs are only a suggestion. You can use whatever you like.
# Make sure to provide your results using the original IDs and not the training IDs.
# Note that many IDs are ignored in evaluation and thus you never need to predict these!

labels = [
    #       name                          id    trainId        hasInstances   ignoreInEval     color
    Label(  'unlabeled'                 ,  0 ,      255       , False        , True         , (  0,  0,  0) ),
    Label(  'medical_chairs'            ,  1 ,        8       , True         , False        , (119, 11, 32) ),
    Label(  'medical_tables'            ,  2 ,        1       , True         , False        , (32,  11, 119)),
    Label(  'medical_equipment_storage' ,  4 ,        2       , True         , False        , (35, 142, 107)),
    Label(  'surgical_beds'             ,  5 ,        3       , True         , False        , (111, 74,  0) ),
    Label(  'medical_beds'              ,  6 ,        3       , True         , False        , ( 81,  0, 81) ),
    Label(  'cutting_tools'             ,  7 ,        4       , True         , False        , (128, 64,128) ),
    Label(  'grasping_tools'            ,  8 ,        4       , True         , False        , (244, 35,232) ),
    Label(  'support_tools'             ,  9 ,        4       , True         , False        , (250,170,160) ),
    Label(  'measuring_tools'           , 10 ,        4       , True         , False        , (230,150,140) ),
    Label(  'imaging'                   , 11 ,        5       , True         , False        , ( 70, 70, 70) ),
    Label(  'life_support'              , 12 ,        6       , True         , False        , (102,102,156) ),
    Label(  'surgical_tools'            , 13 ,        7       , True         , False        , (190,153,153) ),
    Label(  'medical_utitlity_equipment', 14 ,        7       , True         , False        , (180,165,180) ),
    Label(  'general_chairs'            , 15 ,        8       , True         , False        , (150,100,100) ),
    Label(  'cabinets'                  , 16 ,        9       , True         , False        , (150,120, 90) ),
    Label(  'cabinet_doors'             , 17 ,        9       , True         , False        , (153,153,153) ),
    Label(  'bed_accessories'           , 18 ,        3       , True         , False        , (153,153,153) ),
    Label(  'stand'                     , 19 ,       10       , True         , False        , (250,170, 30) ),
    Label(  'healthcare_participants'   , 20 ,       0       , True         , False        , (220, 20, 60) ),
    Label(  'curtain'                   , 21 ,       11       , True         , False        , (107,142, 35) ),
    Label(  'other_tools'               , 22 ,       4       , True         , False        , (152,251,152) ),
    Label(  'furniture_and_decor'       , 23 ,       12       , True         , False        , ( 70,130,180) ),
    Label(  'other_electronics'         , 24 ,       13       , True         , False        , (220, 220, 0) ),
    Label(  'fixtures'                  , 25 ,       14       , True         , False        , (255,  0,  0) ),
    Label(  'office_supplies'           , 26 ,       15       , True         , False        , (  0,  0,142) ),
    Label(  'general_storage'           , 27 ,       2       , True         , False        , (  0,  0, 70) ),
    Label(  'building_elements'         , 28 ,       16       , True         , False        , (  0, 60,100) ),
    Label(  'wall'                      , 29 ,       17       , False        , False        , (  0,  0, 90) ),
    Label(  'floor'                     , 30 ,       18       , False        , False        , (  0,  0,110) ),
    Label(  'ceiling'                   , 31 ,       19       , False        , False        , (  0, 80,100) ),
    Label(  'miscellaneous'             , 32 ,       20       , False        , False        , (  0,  0,230) ),
]



#--------------------------------------------------------------------------------
# Create dictionaries for a fast lookup
#--------------------------------------------------------------------------------

# Please refer to the main method below for example usages!

# name to label object
name2label      = { label.name    : label for label in labels           }
# id to label object
id2label        = { label.id      : label for label in labels           }
# trainId to label object
trainId2label   = { label.trainId : label for label in reversed(labels) }
id2trainId = {label.id      : label.trainId for label in labels }
names = ['healthcare_participants', 'medical_tables', 'medical_equipment_storage', 'medical_beds', 'medical_tools', 'imaging', 'life_support', 'medical_equipment', 'cabinets', 'stand', 'curtain', 'medical_chairs', 'furniture_and_decor', 'other_electronics', 'fixtures', 'office_supplies', 'building_elements', 'wall', 'floor', 'ceiling', 'miscellaneous']
# pal = [name2label[name].color for name in names]
# print (pal)
#--------------------------------------------------------------------------------
# Assure single instance name
#--------------------------------------------------------------------------------

# returns the label name that describes a single instance (if possible)
# e.g.     input     |   output
#        ----------------------
#          car       |   car
#          cargroup  |   car
#          foo       |   None
#          foogroup  |   None
#          skygroup  |   None
def assureSingleInstanceName( name ):
    # if the name is known, it is not a group
    if name in name2label:
        return name
    # test if the name actually denotes a group
    if not name.endswith("group"):
        return None
    # remove group
    name = name[:-len("group")]
    # test if the new name exists
    if not name in name2label:
        return None
    # test if the new name denotes a label that actually has instances
    if not name2label[name].hasInstances:
        return None
    # all good then
    return name

#--------------------------------------------------------------------------------
# Main for testing
#--------------------------------------------------------------------------------

# just a dummy main
if __name__ == "__main__":
    # Print all the labels
    print("List of cityscapes labels:")
    print("")
    print("    {:>28} | {:>3} | {:>7} | {:>12} | {:>12}".format( 'name', 'id', 'trainId', 'hasInstances', 'ignoreInEval' ))
    print("    " + ('-' * 98))
    for label in labels:
        print("    {:>28} | {:>3} | {:>7} | {:>12} | {:>12}".format( label.name, label.id, label.trainId, label.hasInstances, label.ignoreInEval ))
    print("")

    print("Example usages:")

    # Map from name to label
    name = 'healthcare_participants'
    id   = name2label[name].id
    print("ID of label '{name}': {id}".format( name=name, id=id ))

    # Map from trainID to label
    trainId = 0
    name = trainId2label[trainId].name
    print("Name of label with trainID '{id}': {name}".format( id=trainId, name=name ))

    print([trainId2label[id].name for id in range(25)])
    print([trainId2label[id].color for id in range(25)])
  